#ifndef INCL_WAPPUSHD_DBUS
#define INCL_WAPPUSHD_DBUS

/*
 This file is part of wappushd-dev, WAP push daemon development header files.

 Copyright (C) 2008-2009 Nokia Corporation. All rights reserved.

 Contact: Hannu Mallat <hannu.mallat@nokia.com>

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name of Nokia Corporation nor the names of its contributors
    may be used to endorse or promote products derived from this software
    without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
*/

/**
@file wappushd-dbus.h D-Bus API

This is an internal API and considered alpha quality. It may change without
further notice.
@{ */

/** D-BUS service */
#define WAPPUSH_DBUS_SERVICE		"com.nokia.wappush"

/** D-BUS API interface */
#define WAPPUSH_DBUS_INTERFACE          "com.nokia.wappush"

/** Well-known path for D-BUS API */
#define WAPPUSH_DBUS_PATH               "/com/nokia/wappush"

/* Requests related to registration */

/** Register a handler.
 * 
 * Register a handler application for push messages of a given type. 
 * The handler application must implement a D-BUS interface for receiving
 * forwarded push messages (see #WAPPUSH_HANDLER_INTERFACE and 
 * #WAPPUSH_HANDLER_METHOD). 
 *
 * The registration information is stored in persistent memory, so an 
 * application needs to do it only once. 
 *
 * Arguments:
 * <pre>
 *  DBUS_TYPE_STRING            push application ID URN (see OMNA list)
 *  DBUS_TYPE_STRING            D-BUS service of the handler application
 *  DBUS_TYPE_STRING            D-BUS object path of the handler application</pre>
 * Return arguments:
 * <pre>
 *  none</pre>
 */
#define WAPPUSH_REGISTER_REQ		"register"

/** Deregister a handler. 
 * 
 * Arguments: 
 * <pre>
 *  DBUS_TYPE_STRING            push application ID URN</pre>
 * Return arguments:
 * <pre>
 *  none</pre>
 */
#define WAPPUSH_DEREGISTER_REQ          "deregister"

/* Request errors */

/** Cannot register application due to already existing registration */
#define WAPPUSH_ERROR_ALREADY_REGISTERED "com.nokia.wappush.error.already_registered"

/** Cannot deregister application since not registration for message type exists */
#define WAPPUSH_ERROR_NOT_REGISTERED "com.nokia.wappush.error.not_registered"

/** Interface that the registered application has to implement */
#define WAPPUSH_HANDLER_INTERFACE	"com.nokia.WAPPushHandler"

/** Handle a push message.
 * 
 * The push message handler application has to implement this method
 * in order to be able to receive messages over D-BUS.
 *
 * Arguments:
 * <pre>
 *  DBUS_TYPE_BYTE              Bearer type, either IP (UDP) or SMS. The 
 *                              following arguments depend on the type. 
 * 
 *  For IP bearer: 
 *
 *  DBUS_TYPE_ARRAY (BYTE)      Source IP address in network byte order
 *  DBUS_TYPE_ARRAY (BYTE)      Destination IP address, as above
 *  DBUS_TYPE_UINT16            Source UDP port in network byte order
 *  DBUS_TYPE_UINT16            Destination UDP port in network byte order
 *  DBUS_TYPE_ARRAY (BYTE)      WSP headers
 *  DBUS_TYPE_ARRAY (BYTE)      WSP payload
 *
 *  For SMS bearer:
 *
 *  DBUS_TYPE_STRING            Originating address
 *  DBUS_TYPE_INT32             Source application port
 *  DBUS_TYPE_INT32             Destination application port
 *  DBUS_TYPE_ARRAY (BYTE)      WSP headers
 *  DBUS_TYPE_ARRAY (BYTE)      WSP payload</pre>
 * Return arguments:
 * <pre>
 *  none</pre>
 */
#define WAPPUSH_HANDLER_METHOD		"HandleWAPPush"

#define WAPPUSH_BEARER_TYPE_IP	0x00 /**< IP bearer type */
#define WAPPUSH_BEARER_TYPE_SMS	0x01 /**< SMS bearer type */

/** Special application ID URN for client provisioning messages. 
 *
 * Client provisioning push messages do not contain a WAP application
 * ID header, so a handler for those has to be registered using a
 * special URN not in the OMNA list. 
 */
#define WAPPUSH_CLIENT_PROVISIONING_APPID "x-oma-cp" 

/** @} */

#endif
